<?php
/**
 * Plugin Name: Custom Shortcode Sidebars
 * Description: Create custom sidebars that are called via shortcodes in your post and pages.
 * Version: 1.0
 * Revision Date: August 12, 2010
 * Requires at least: WP 2.5
 * Tested up to: WP 3.0.1
 * Author: Jason Grim
 * Author URI: http://jgwebdevelopment.com
 * Plugin URI: http://jgwebdevelopment.com/plugins/custom-sidebars-shortcode
 */

// Get current custom sidebars
$cwsc_current_widgets = get_option ( 'cwsc_widgets' );

// If there are some current ones, then loop through and register them
if ( is_array( $cwsc_current_widgets ) && ! empty( $cwsc_current_widgets ) ) {
	cwsc_register_sidebars( $cwsc_current_widgets );
}

/**
 * Filter function that execute on each shortcode in content
 * 
 * [mysidebar id='the-id-of-the-custom-sidebar']
 * 
 * @param array $atts
 * @since 1.0
 * @return string The widget HTML
 */
function cwsc_shortcode_filter ( $atts ) {
	extract ( shortcode_atts ( array (
		'id' => null 
	), $atts ) );
	
	if ( is_null( $id ) ) return null;
	
	ob_start ();
	dynamic_sidebar ( $id );
	$the_widget = ob_get_contents ();
	ob_end_clean ();
	
	return $the_widget;
}
add_shortcode ( 'mysidebar', 'cwsc_shortcode_filter' );

/**
 * Loops through all the registered side bars and registers them
 * 
 * @since 1.0
 * @return null
 */
function cwsc_register_sidebars ( $sidebars = array() ) {
	
	if ( ! is_array ( $sidebars ) || empty( $sidebars ) ) return false;
	
	foreach ( $sidebars as $sidebar_id => $sidebar_values ) {
		register_sidebar ( array (
			'name' => $sidebar_values ['name'], 
			'id' => $sidebar_id, 
			'description' => $sidebar_values ['description'], 
			'before_widget' => $sidebar_values ['before_widget'], 
			'after_widget' => $sidebar_values ['after_widget'], 
			'before_title' => $sidebar_values ['before_title'], 
			'after_title' => $sidebar_values ['after_title'] 
		) );
	}
}

/**
 * Creates a slug friendly version of the input string
 * 
 * @param string $title
 * @since 1.0
 * @return string Conferted string to slug format
 */
function cwsc_slug_maker ( $title = null ) {
	
	// Only use standard characters
	$slug = preg_replace ( "/[^a-zA-Z0-9 ]/", "", $title );
	// Remove all double spaces
	$slug = str_replace ( "  ", " ", $slug );
	// Replace double spaced with URL friendly slash
	$slug = str_replace ( " ", "-", $slug );
	// Make the slug all lowercase
	$slug = strtolower ( $slug );
	
	return $slug;
}

/**
 * Loops through all the sidebars and echos the table row in the admin page
 * 
 * @since 1.0
 * @return null
 */
function cwsc_loop_current_widgets () {
	$sidebars = get_option ( 'cwsc_widgets' );
	
	if ( ! is_array( $sidebars ) || empty ( $sidebars ) ) return null;
	
	foreach ( $sidebars as $sidebar_id => $sidebar_values ) {
	
	?>	
	<tr class="iedit alt" id="recipe-cat-<?php echo $cat->cat_id; ?>">
		<td class="name column-name"><a href="#"> <?php echo $sidebar_values ['name']; ?> </a><br>
		<div class="row-actions">
			<span class="edit"><a href="?page=custom-widget-shortcode&cwsc-edit-widget=<?php echo $sidebar_id; ?>">Edit</a></span>
			<span class="delete"> | <a
			class="delete:the-list:link-cat-<?php echo $sidebar_id; ?> submitdelete"
			onclick="return confirm('Warning: You are about to delete the &quot;<?php echo $sidebar_values ['name']; ?>&quot; sidebar.  Are you sure?')"
			href="?page=custom-widget-shortcode&cwsc-delete-widget=<?php echo $sidebar_id; ?>">Delete </a> </span>
		</div>
		</td>
		<td class="slug column-slug">[mysidebar id="<?php echo $sidebar_id; ?>"]</td>
	</tr>
	<?php
	
	}
}

/**
 * The add menu function thingy
 * 
 * @since 1.0
 * @return null
 */
function cwsc_register_page () {
	add_options_page ( 'Custom Sidebar Shortcode', 'Custom Sidebars', 'manage_options', 'custom-widget-shortcode', 'cwsc_page' );
}
add_action ( 'admin_menu', 'cwsc_register_page' );

/**
 * The page in the admin menu to add/edit/delete sidebars
 */
function cwsc_page () {
	
	echo '<h2>Manage Custom Sidebars</h2>';
	
	// default values
	$current_widget = array(
						'name' => null,
						'description' => null,
						'before_widget' => null,
						'after_widget' => null,
						'before_title' => null,
						'after_title' => null,
						'id' => null
					);
	
	// add new widget
	if ( isset( $_REQUEST ['cwsc-add-widget'] ) ) {
		
		if ( is_null( $_REQUEST ['widget'] ['name'] ) || empty( $_REQUEST ['widget'] ['name'] ) ) {
			echo '<div class="error"><p>Sidebar name is required!</p></div>';
			$current_widget = $_REQUEST ['widget'];
		} else {
		
			$sidebars = get_option ( 'cwsc_widgets', array() );
			
			$new_sidebar_id = cwsc_slug_maker( $_REQUEST ['widget'] ['name'] );
			
			$new_widget_values = $_REQUEST ['widget'];
			
			$sidebars [ $new_sidebar_id ] = $new_widget_values;
			
			update_option( 'cwsc_widgets', $sidebars );
			
			echo '<div class="updated"><p>Custom Sidebar Saved!</p></div>';
		}
	}
	
	// edit widget	
	if ( isset( $_REQUEST ['cwsc-edit-widget'] ) ) {
		$sidebars = get_option ( 'cwsc_widgets' );
		
		$current_widget = $sidebars [ $_REQUEST ['cwsc-edit-widget'] ];
	}
	
	// delete widget
	if ( isset( $_REQUEST ['cwsc-delete-widget'] ) ) {
		$sidebars = get_option ( 'cwsc_widgets' );
		
		unset( $sidebars [ $_REQUEST ['cwsc-delete-widget'] ] );
		
		update_option( 'cwsc_widgets', $sidebars );
		
		echo '<div class="updated"><p>Custom Sidebar Removed!</p></div>';
	}
	
	?>
<div class="wrap">
	<div id="col-container">
	<div id="col-right">
	<div class="col-wrap">
		<form action="#" method="get" id="posts-filter">
		<h3>Existing Custom Sidebars</h3>
			<table cellspacing="0" class="widefat fixed">
				<thead>
					<tr class="alt">
						<th>Name</th>
						<th>Shortcode</th>
					</tr>
				</thead>
				<tfoot>
					<tr>
						<th>Name</th>
						<th>Shortcode</th>
					</tr>
				</tfoot>
				<tbody>
					<?php echo cwsc_loop_current_widgets(); ?>
				</tbody>
			</table>
		</form>
	</div>
	
	</div>
		<div id="col-left">
			<div class="col-wrap">
				<div class="form-wrap">
				<h3>Custom Sidebar</h3>
					<form enctype="multipart/form-data" name="sc" id="sc" class="add:the-list: validate" method="post" action="?page=custom-widget-shortcode">
						<div class="form-field form-required">
							<label for="name">Sidebar Name</label>
							<input type="text" name="widget[name]" id="name" value="<?php echo $current_widget ['name']; ?>" size="40">
							<p>Title for your sidebar in the widget menu.</p>
						</div>
						
						<div class="form-field">
							<label for="description">Description</label>
							<input type="text" name="widget[description]" id="description" value="<?php echo $current_widget ['description']; ?>" size="40">
							<p>Describe what this custom sidebar will be used for and maybe it's placement for reference later.</p>
						</div>
						
						<div class="form-field">
							<label for="before_widget">Before Widget</label>
							<input type="text" name="widget[before_widget]" id="before_widget" value="<?php echo $current_widget ['before_widget']; ?>" size="40">
							<p>Text or HTML to be displayed before each widget in this sidebar.</p>
						</div>
						
						<div class="form-field">
							<label for="after_widget">After Widget</label>
							<input type="text" name="widget[after_widget]" id="after_widget" value="<?php echo $current_widget ['after_widget']; ?>" size="40">
							<p>Text or HTML to be displayed after each widget in this sidebar.</p>
						</div>
						
						<div class="form-field">
							<label for="before_title">Before Title</label>
							<input type="text" name="widget[before_title]" id="before_title" value="<?php echo $current_widget ['before_title']; ?>" size="40">
							<p>Text or HTML to be displayed before the titles of each widget in this sidebar.</p>
						</div>
						
						<div class="form-field">
							<label for="after_title">After Title</label>
							<input type="text" name="widget[after_title]" id="after_title" value="<?php echo $current_widget ['after_title']; ?>" size="40">
							<p>Text or HTML to be displayed after the titles of each widget in this sidebar.</p>
						</div>
						
						<p class="submit"><input type="submit" class="button" name="cwsc-add-widget" value="Save Custom Widget"></p>
					</form>
				</div>
			</div>
		</div>
	</div>
</div>
<?php
}